import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

/// 常用的 widget
///

class Blank {
  Blank._();

  static _BlankVertical get vertical => _BlankVertical(height: 10);

  static _BlankHorizontal get horizontal => _BlankHorizontal(width: 10);
}

class _BlankVertical extends SizedBox {
  _BlankVertical({double? height}) : super(height: height);

  SizedBox operator *(double space) => SizedBox(height: space * 10);
}

class _BlankHorizontal extends SizedBox {
  _BlankHorizontal({double? width}) : super(width: width);

  SizedBox operator *(double space) => SizedBox(width: space * 10);
}

/// 输入限制--全数字
class NumberFormatter extends TextInputFormatter {
  bool isNumber(String value) {
    return RegExp('^[0-9]*\$').hasMatch(value);
  }

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    String value = newValue.text;
    return isNumber(value) ? newValue : oldValue;
  }
}

/// 文本
class TextWidget extends StatelessWidget {
  final String text;
  final double? width;
  final double? height;
  final Color? color;
  final Alignment? alignment;
  final double? fontSize;
  final EdgeInsetsGeometry? margin;
  final EdgeInsetsGeometry? padding;
  final Decoration? decoration;
  final Map<AxisDirection, Widget>? icon;
  final Map<AxisDirection, double>? iconMargin;
  final Function? onClick;

  TextWidget(this.text, {Key? key, this.width, this.height, this.color, this.alignment, this.fontSize, this.margin, this.padding, this.decoration, this.icon, this.iconMargin, this.onClick}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    Widget left = icon?[AxisDirection.left] ?? Container(width: 0, height: 0);
    Widget right = icon?[AxisDirection.right] ?? Container(width: 0, height: 0);
    Widget up = icon?[AxisDirection.up] ?? Container(width: 0, height: 0);
    Widget down = icon?[AxisDirection.down] ?? Container(width: 0, height: 0);
    double pl = iconMargin?[AxisDirection.left] ?? 0;
    double pr = iconMargin?[AxisDirection.right] ?? 0;
    double pd = iconMargin?[AxisDirection.down] ?? 0;
    double pu = iconMargin?[AxisDirection.up] ?? 0;
    var tv = Container(
      constraints: BoxConstraints(minHeight: height ?? 0, minWidth: width ?? 0, maxWidth: width ?? double.infinity, maxHeight: height ?? double.infinity),
      margin: margin,
      padding: padding,
      alignment: alignment,
      decoration: decoration,
      child: Row(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          left,
          pl == double.infinity ? Spacer() : Padding(padding: EdgeInsets.symmetric(horizontal: pl)),
          Column(
            mainAxisSize: MainAxisSize.min,
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              up,
              pu == double.infinity ? Spacer() : Padding(padding: EdgeInsets.symmetric(vertical: pu)),
              Text(text, style: TextStyle(fontSize: fontSize, color: color)),
              pd == double.infinity ? Spacer() : Padding(padding: EdgeInsets.symmetric(vertical: pd)),
              down,
            ],
          ),
          pr == double.infinity ? Spacer() : Padding(padding: EdgeInsets.symmetric(horizontal: pr)),
          right,
        ],
      ),
    );
    return onClick == null ? tv : InkWell(child: tv, onTap: () => onClick?.call());
  }
}

/// 虚线
class DashLine extends StatelessWidget {
  final double height;
  final Color color;
  final double dashWidth;

  const DashLine({this.height = 1, this.color = Colors.grey, this.dashWidth = 10});

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        final boxWidth = constraints.constrainWidth();
        final dashHeight = height;
        final dashCount = (boxWidth / (2 * dashWidth)).floor();
        return Flex(
          children: List.generate(dashCount, (_) => SizedBox(width: dashWidth, height: dashHeight, child: DecoratedBox(decoration: BoxDecoration(color: color)))),
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          direction: Axis.horizontal,
        );
      },
    );
  }
}

/// 圆点
class Dot extends StatelessWidget {
  final double radius;
  final Color color;

  const Dot(this.radius, this.color);

  @override
  Widget build(BuildContext context) {
    return Container(width: radius, height: radius, decoration: BoxDecoration(color: color, borderRadius: BorderRadius.circular(radius)));
  }
}



//  style

List<BoxShadow> get boxShadowGray => <BoxShadow>[BoxShadow(color: Colors.grey.shade200, blurRadius: 10, offset: Offset(0, 5))];
