import 'dart:convert';

import 'package:flutter/material.dart';

extension WidgetExt on Widget {
  /// 默认有效点击间隔一秒，interval=0 时无间隔时间
  InkWell onTap(VoidCallback onTap, {int? interval, bool? releaseFocus = true}) => _InkWell._(this, [DateTime.now().millisecondsSinceEpoch], interval, onTap, releaseFocus);

  InkWell onLongPress(VoidCallback onLongPress) => InkWell(onLongPress: onLongPress, child: this);

  Expanded expanded({int flex = 1}) => Expanded(child: this, flex: flex);

  @deprecated
  Padding padding(EdgeInsetsGeometry padding) => Padding(padding: padding, child: this);

  Padding extPadding(EdgeInsetsGeometry padding) => Padding(padding: padding, child: this);

  @deprecated
  Container margin(EdgeInsetsGeometry margin) => Container(child: this, margin: margin);

  Container extMargin(EdgeInsetsGeometry margin) => Container(child: this, margin: margin);

  Positioned positioned({double? left, double? top, double? right, double? bottom}) => Positioned(child: this, left: left, top: top, right: right, bottom: bottom);

  SizedBox size(double width, double height) => SizedBox(width: width, height: height, child: this);

  @deprecated
  Container container({double? width = double.infinity, double? height = double.infinity, Decoration? decoration}) => Container(width: width, height: height, child: this, alignment: Alignment.center, decoration: decoration);

  Container extContainer({double? width = double.infinity, double? height = double.infinity, Decoration? decoration}) => Container(width: width, height: height, child: this, alignment: Alignment.center, decoration: decoration);

  Center center({double? width = double.infinity, double? height = double.infinity}) => Center(child: this);

  SingleChildScrollView singleScrollView({Axis scrollDirection = Axis.vertical}) => SingleChildScrollView(child: this, scrollDirection: scrollDirection);

  Widget visible(Visible visible) => visible == Visible.VISIBLE ? this : (visible == Visible.INVISIBLE ? Opacity(opacity: 0, child: this) : Offstage(offstage: true, child: this));

  Opacity alpha(double opacity) => Opacity(child: this, opacity: opacity);

  SliverToBoxAdapter sliverAdapter({Key? key}) => SliverToBoxAdapter(child: this, key: key);

  ClipRRect circular(double radius) => ClipRRect(child: this, borderRadius: BorderRadius.circular(radius));
}

enum Visible { VISIBLE, INVISIBLE, GONE }

class Debounce {
  Debounce._();

  static int _last = 0;

  static VoidCallback onTap(VoidCallback onTap, {int? interval = 1000, bool? releaseFocus = true}) => () {
        var millisecondsSinceEpoch = DateTime.now().millisecondsSinceEpoch;
        if ((millisecondsSinceEpoch - _last) > (interval ?? 1000)) {
          onTap.call();
          if (releaseFocus == true) {
            FocusManager.instance.primaryFocus?.unfocus();
          }
          _last = millisecondsSinceEpoch;
        }
      };

  /// 点击二次确认
  static int _doubleClickConfirmation = 0;

  static Future<bool> doubleClickConfirmation({int? interval = 1000, VoidCallback? first, VoidCallback? second}) {
    int now = DateTime.now().millisecondsSinceEpoch;
    if (now - _doubleClickConfirmation > (interval ?? 1000)) {
      _doubleClickConfirmation = DateTime.now().millisecondsSinceEpoch;
      first?.call();
      return Future.value(false);
    } else {
      second?.call();
      return Future.value(true);
    }
  }
}

extension IntervalVoidCallback on VoidCallback {
  VoidCallback debounce({int interval = 1000, bool? releaseFocus = true}) => Debounce.onTap(this, interval: interval, releaseFocus: releaseFocus);
}

class _InkWell extends InkWell {
  final Widget? child;

  _InkWell(GestureTapCallback onTap, {Key? key, this.child}) : super(key: key, child: child, onTap: onTap);

  factory _InkWell._(Widget? child, List<int>? last, int? interval, GestureTapCallback onTap, bool? releaseFocus) {
    return _InkWell(Debounce.onTap(onTap, interval: interval, releaseFocus: releaseFocus), child: child);
  }
}

extension WebViewHTMLCover on String {
  /// 富文本转换
  String get coverHtml {
    var html = """
    <html>
            <head>
              <meta charset="UTF-8">
            </head>
            <body  onload="SendHight()">
              $this
            </body>
        </html>
    """;
    String initialUrl = "data:text/html;charset=utf-8;base64,${base64Encode(const Utf8Encoder().convert(html))}";
    return initialUrl;
  }

  /// WebView 动态高度
/**
    WebView(
    initialUrl: "url",
    javascriptMode: JavascriptMode.unrestricted,
    onWebViewCreated: (controller) => _controller = controller,
    onPageFinished: (s) async {
    final scrollHeight = await _controller?.evaluateJavascript(
    '(() => document.body.scrollHeight)();',
    );
    var _webViewHeight= double.parse(scrollHeight!);
    },
    )
 */
}
